#perl runDiscovery.pl -h imda4rh65:8581 -n "First Discovery" -u admin -p admin
#!/usr/bin/perl
#runDiscovery.pl
# Author - Seenu Mathew - seenu.mathew@ca.com
# Prerequisites: tested with perl 5.18
#	typically, REST::Client and XML::XSLT modules must be installed (cpan)
#  
# DISCLAIMER
# This script is provided AS IS and no warranties are provided or made.  
# For more information refer to http://www.ca.com/us/legal/terms-for-user-sites.aspx
=pod

Perl script to run discovery using command line 

This script can be run from any machine which has Strawberry perl and a valid connection to Data Aggregator using rest client.

1.	Issue aHTTP PUT to http://<dahost>8581/rest/tenant/<daTenantId>/discoveryprofiles/<daDiscoveryProfileId> with the following content:
Body:
	<DiscoveryProfile version="1.0.0">
	  <RunStatus>START</RunStatus>
	</DiscoveryProfile>
2.	Print discovery status as Sucess if we get a "200 OK" in the PUT


Assumption
==========
1. The tenant is default tenant and its id is 1. 
2. If you would like to run a this script for a different tenant, find the Tenant id by issuing  a GET on 
http://<DAhost>:8581/rest/tenants
3. Then use the -tenant attribute to put in the correct tenant id

Run the discovery profile
1.	Issue a HTTP PUT to http://<dahost>8581/rest/tenant/<daTenantId>/discoveryprofiles/<daDiscoveryProfileId> with the following content:
Body:
	<DiscoveryProfile version="1.0.0">
	  <RunStatus>START</RunStatus>
	</DiscoveryProfile>
	
Check the status of a discovery
You can periodically check the status of a running discovery. 
1.	Issue an HTTP GET to http://<dahost>8581/rest/discoveryprofiles/<daDiscoveryProfileId>. 
2.	Check the RunStatus value in the response. A READY or SCHEDULED status means that the discovery is complete.

=cut

use strict;
use warnings;
use Data::Dumper;
use Getopt::Std;
use REST::Client;
use XML::Simple;
use MIME::Base64;

$| = 1;

sub main() {

	my %opt;
	getopts( 'h:n:u:p: v', \%opt );

	my $hostname = $opt{h};
	my $nameDiscProf = $opt{n};
	my $user = $opt{u};
	my $pass = $opt{p};
		
	checkusage($hostname, $nameDiscProf, $user, $pass);
	createXmlFile( $hostname, $nameDiscProf, $user, $pass);
	
}

sub checkusage(){
	
	my $hostname = shift;
	my $nameDiscProf = shift;
	my $user = shift;
	my $pass = shift;
	
	unless(defined($hostname) and defined ($nameDiscProf) and defined($hostname) and defined($pass)) {
		usage();
	}
	else {
	 
	}

}

sub usage() {

	my $help = <<USAGE;
	Usage of getDiscProfileList.pl is as follows
	
	-h <hostname or ip address of CA Data Aggregator + port> eg: dahost:8581
	-u <Username that has access to DA (usually admin)>
	-p <Password>
	-n <Name of Discovery Profile>
	
	
Example: 
1. 	perl runDiscovery.pl -h DataAggregatorHostname:8581 -user admin -pass admin -n "Routers DiscProfile""
	The above command will run the discovery profile with the name RoutersDiscProfile on the default tenant with id 1
	


USAGE

	die $help;

}

sub createXmlFile() {

	my $hostname = shift;
	my $nameDiscProf = shift;
	my $user = shift;
	my $p = shift;
	my $tenant_id = 1;
	my $id =0;
	my $id1 = 0;
	
	
	my $filename = "daxmlfile"; 
	my $url = $hostname . '/rest';
	my $message = "\<DiscoveryProfile version=\"1.0.0\"\> \<RunStatus\>START<\/RunStatus> <\/DiscoveryProfile>";

	my $client = REST::Client->new( host => $url );
	$client->GET('/discoveryprofiles');

	open( XML, "> " . $filename . ".xml" )
	  or die "error when creating outputfile '$filename'.xml: $!\n";
	print XML $client->responseContent() . "\n";
	close(XML);
	$url = "http://" . $url .  "/tenant/". $tenant_id . "/discoveryprofiles";
	 
	
	
	#print Dumper($data);
	
	my $xml = new XML::Simple;
	my $data = $xml->XMLin( $filename . ".xml",ForceArray => 1);
	
	#my $request = HTTP::Request->new(GET => '$url . "/discoveryprofiles"');
	
	#my $ua = LWP::UserAgent->new;
	#my $response = $ua->request($request);
	#print $response;
	
	#my $r = HTTP::Request->new(GET, )
	
		foreach my $discProfile ( @{ $data->{DiscoveryProfile} } ) {
		
		my $item = $discProfile->{Item};
		$id = $discProfile->{ID}->[0];
		#As Item is an arry, We have to traverse through the array and print. 
		#There is only one name and create time therefore [0] is being used
		
		
				foreach my $attr (@{$item}){
				#print Dumper($attr);
				my $name = $attr->{Name}->[0];
				#print $attr->{CreateTime}->[0] . ",";
				
				
					if ($name eq $nameDiscProf){
						#print $nameDiscProf;
						$id1 = $id;
														
					}
				
				}
		}
		
		print "Discovery process started for" . $nameDiscProf . "\n";
		my $discProfid = "/" . $id1;
		my $client1 = REST::Client->new();
		$client1->setHost($url);
		$client1->addHeader('Content-Type', 'application/xml');
		$client1->addHeader('Charset', 'UTF-8');
		$client1->addHeader("Authorization", "Basic " . encode_base64("$user:$p"));
		$client1->setTimeout(10);
		#$client1->GET($discProfid);
		$client1->PUT($discProfid, $message);
		if ($client1->responseCode() == 200){
			print "Discovery complete". "\n";
		}	
		
		
		
}
main();
